/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package foo.bar;

import org.apache.camel.Exchange;
import org.apache.camel.Processor;
import org.apache.camel.builder.RouteBuilder;
import org.apache.xbean.classloader.NamedClassLoader;

/**
 * A Camel Router
 *
 * @version $Revision: 1.1 $
 */
public class MyRouteBuilder extends RouteBuilder {

    public void configure() {
    	from("timer://classloader-example?fixedRate=true&delay=3000&period=10000")
    	.setBody(constant("<message>Hello world!</message>"))
    	.process(new Processor() {
			public void process(Exchange arg0) throws Exception {
				loadMyClass();  // SU ClassLoader
			}
    	})
    	
    	.to("jbi:endpoint:http://test/MyProviderService/myProvider")
    	// after sending to a JBI endpoint, the context ClassLoader is replaced with the 
    	// Component ClassLoader preventing further code from loading classes in the SU lib
    	.process(new Processor() {
			public void process(Exchange arg0) throws Exception {
				loadMyClass();  // Component ClassLoader
			}
    	}); 

    }
    
    private void loadMyClass() {
    	NamedClassLoader cl = (NamedClassLoader) Thread.currentThread().getContextClassLoader();
    	System.out.println("Loading foo.bar.MyClass with ClassLoader [" +cl.getName()+ "]...");
		try {
			Class.forName("foo.bar.MyClass", true, cl);
			System.out.println("Success!\n");
		} catch (ClassNotFoundException e) {
			System.out.println("Failed to load foo.bar.MyClass!\n");
		}
    }
}
